@@ -85,58 +85,66 @@
 database_type='Psycopg'
 __doc__='''%s Database Connection
 
-$Id: DA.py,v 1.15.2.3 2002/06/17 23:45:00 fog Exp $''' % database_type
-__version__='$Revision: 1.15.2.3 $'[11:-2]
+$Id: DA.py,v 1.20.2.11 2004/05/21 08:33:37 fog Exp $''' % database_type
+__version__='$Revision: 1.20.2.11 $'[11:-2]
+__psycopg_versions__ = ('1.1.12',)
+
 
 from db import DB
 import Shared.DC.ZRDB.Connection, sys, DABase, time
-from Globals import HTMLFile
-from ImageFile import ImageFile
+from Globals import HTMLFile, ImageFile
 from ExtensionClass import Base
-from string import find,join,split
+from string import find, join, split, rindex
 try:
     import psycopg
     from psycopg import new_type, register_type, DATETIME, TIME, DATE, INTERVAL
-    print "psycopg version is", psycopg.__version__
 except StandardError, err:
     print err
 try:
     from DateTime import DateTime
-    print "datetime object:", repr(DateTime)
 except StandardError, err:
     print err
-
 try:
     from App.Dialogs import MessageDialog
 except:
     pass
 
-
 manage_addZPsycopgConnectionForm = HTMLFile('connectionAdd', globals())
 def manage_addZPsycopgConnection(self, id, title,
                                  connection_string, zdatetime=None,
-                                 check=None, REQUEST=None):
+				 tilevel=2, check=None, REQUEST=None):
     """Add a DB connection to a folder"""
     self._setObject(id, Connection(id, title, connection_string, zdatetime,
-                                   check))
+                                   check, tilevel))
     if REQUEST is not None: return self.manage_main(self,REQUEST)
 
 
 # Convert an ISO timestamp string from postgres to a DateTime(zope version)
 # object.
 def cast_DateTime(str):
-    return DateTime(str[:-3])
+    if str:
+        # this will split us into [date, time, GMT/AM/PM(if there)]
+        dt = split(str, ' ')
+        if len(dt) > 1:
+            # we now should split out any timezone info
+            dt[1] = split(dt[1], '-')[0]
+            dt[1] = split(dt[1], '+')[0]
+            return DateTime(join(dt[:2], ' '))
+        else:
+            return DateTime(dt[0])
 
 # Convert an ISO date string from postgres to a DateTime(zope version)
 # object.
 def cast_Date(str):
-    return DateTime(str)
+    if str:
+        return DateTime(str)
 
 # Convert a time string from postgres to a DateTime(zope version) object.
 # WARNING: We set the day as today before feeding to DateTime so
 # that it has the same DST settings.
 def cast_Time(str):
-    return DateTime(time.strftime('%Y-%m-%d %H:%M:%S',
+    if str:
+        return DateTime(time.strftime('%Y-%m-%d %H:%M:%S',
                                   time.localtime(time.time())[:3]+
                                   time.strptime(str[:8], "%H:%M:%S")[3:]))
 
@@ -154,25 +162,29 @@ class Connection(DABase.Connection):
     meta_type = title = 'Z %s Database Connection' % database_type
     icon = 'misc_/Z%sDA/conn' % database_type
 
-    def __init__(self, id, title, connection_string, zdatetime, check=None):
+    def __init__(self, id, title, connection_string, zdatetime,
+                 check=None, tilevel=2):
         self.zdatetime=zdatetime
         self.id=str(id)
-        self.edit(title, connection_string, zdatetime, check)
+        self.edit(title, connection_string, zdatetime,
+                  check=check, tilevel=tilevel)
 
-    def edit(self, title, connection_string, zdatetime, check=1):
+    def edit(self, title, connection_string, zdatetime, check=1, tilevel=2):
         self.title=title
         self.connection_string=connection_string
         self.zdatetime=zdatetime
+	self.tilevel=tilevel
         self.set_type_casts()
         if check: self.connect(connection_string)
 
     manage_properties=HTMLFile('connectionEdit', globals())
 
     def manage_edit(self, title, connection_string,
-                    zdatetime=None, check=None, REQUEST=None):
+                    zdatetime=None, check=None, tilevel=2, REQUEST=None):
         """Change connection
         """
-        self.edit(title, connection_string, zdatetime, check=None)
+        self.edit(title, connection_string, zdatetime,
+                  check=check, tilevel=tilevel)
         if REQUEST is not None:
             return MessageDialog(
                 title='Edited',
@@ -184,7 +196,7 @@ class Connection(DABase.Connection):
         "Make changes to psycopg default typecast list"
         if self.zdatetime:
             #use zope internal datetime routines
-            ZDATETIME=new_type((1184,), "ZDATETIME", cast_DateTime)
+            ZDATETIME=new_type((1184,1114), "ZDATETIME", cast_DateTime)
             ZDATE=new_type((1082,), "ZDATE", cast_Date)
             ZTIME=new_type((1083,), "ZTIME", cast_Time)
             ZINTERVAL=new_type((1186,), "ZINTERVAL", cast_Interval)
@@ -208,12 +220,22 @@ class Connection(DABase.Connection):
     def connect(self,s):
         try: self._v_database_connection.close()
         except: pass
+
+        # check psycopg version and raise exception if does not match
+        if psycopg.__version__ not in __psycopg_versions__:
+            raise ImportError("psycopg version mismatch: " +
+                              psycopg.__version__)
+
         self.set_type_casts()
         self._v_connected=''
         DB=self.factory()
         try:
             try:
-                self._v_database_connection=DB(s)
+                # this is necessary when upgrading from old installs without
+                # having to recreate the connection object
+                if not hasattr(self, 'tilevel'):
+                    self.tilevel = 2
+                self._v_database_connection=DB(s, self.tilevel)
             except:
                 t, v, tb = sys.exc_info()
                 raise 'BadRequest', (
@@ -227,6 +249,7 @@ class Connection(DABase.Connection):
         return self
 
     def sql_quote__(self, v):
+        # quote dictionary
         quote_dict = {"\'": "''", "\\": "\\\\"}
         for dkey in quote_dict.keys():
             if find(v, dkey) >= 0:
