@@ -83,8 +83,8 @@
 # 
 ##############################################################################
 
-'''$Id: db.py,v 1.29.2.5 2002/06/17 23:45:00 fog Exp $'''
-__version__='$Revision: 1.29.2.5 $'[11:-2]
+'''$Id: db.py,v 1.31.2.6 2004/05/16 08:09:50 fog Exp $'''
+__version__='$Revision: 1.31.2.6 $'[11:-2]
 
 
 from Shared.DC.ZRDB.TM import TM
@@ -101,40 +101,39 @@ from psycopg import NUMBER, STRING, INTEGER, FLOAT, DATETIME
 from psycopg import BOOLEAN, ROWID, LONGINTEGER
 from ZODB.POSException import ConflictError
 
-failures = 0
-calls = 0
-last_call_time = time()
-
 class DB(TM,dbi_db.DB):
 
     _p_oid = _p_changed = _registered = None
 
-    connect = psycopg.connect
-
-    def __init__(self, connection):
+    def __init__(self, connection, tilevel):
         self.connection = connection
+        self.tilevel = tilevel
         self.db = self.connect(self.connection)
+        self.failures = 0
+        self.calls = 0
+
+    def connect(self, connection):
+        o = psycopg.connect(connection)
+        o.set_isolation_level(int(self.tilevel))
+        return o
     
     def _finish(self, *ignored):
-	try:
-            if hasattr(self, 'db') and self.db:
-                self.db.commit()
-            else:
-                self.db = self.connect(self.connection)
-        except psycopg.Error:
-            pass
+        if hasattr(self, 'db') and self.db:
+            self.db.commit()
             
     def _abort(self, *ignored):
-        try:
-            if hasattr(self, 'db') and self.db:
-                self.db.rollback()
-            else:
-                self.db = self.connect(self.connection)
-        except psycopg.Error:
-            pass
-
+        if hasattr(self, 'db') and self.db:
+            self.db.rollback()
+            
+    def _cursor(self):
+        """Obtains a cursor in a safe way."""
+        if not hasattr(self, 'db') or not self.db:
+            self.db = self.connect(self.connection)
+        return self.db.cursor()
+    
     def tables(self, rdb=0, _care=('TABLE', 'VIEW')):
-        c = self.db.cursor()
+        self._register()
+        c = self._cursor()
         c.execute('SELECT t.tablename AS NAME, '
                   '\'TABLE\' AS TYPE FROM pg_tables t '
                   'WHERE tableowner <> \'postgres\' '
@@ -156,9 +155,10 @@ class DB(TM,dbi_db.DB):
         return r
 
     def columns(self, table_name):
-        c = self.db.cursor()
+        self._register()
+        c = self._cursor()
         try:
-            r = c.execute('select * from %s where 1=0' % table_name)
+            r = c.execute('select * from "%s" where 1=0' % table_name)
         except:
             return ()
         desc = c.description
@@ -186,25 +186,24 @@ class DB(TM,dbi_db.DB):
                 'Nullable': 0})
         return r
 
-    def query(self, query_string, max_rows=9999999):
-        global failures, calls, last_call_time
-        calls = calls+1
+    def query(self, query_string, max_rows=None):
+        self._register()
+        self.calls = self.calls+1
+        
         desc = ()
         result = []
-        self._register()
-        c = self.db.cursor()
         nselects = 0
-
+        
+        c = self._cursor()
+        
         try:
             for qs in filter(None, map(strip, split(query_string, '\0'))):
                 try:
                     r = c.execute(qs)
-                except psycopg.ProgrammingError, perr:
-                    sf = \
-                      "ERROR:  Can't serialize access due to concurrent update"
-                    if perr.args[0][:len(sf)] == sf:
+                except (psycopg.ProgrammingError,psycopg.IntegrityError), perr:
+                    if perr.args[0].find("concurrent update") > -1:
                         raise ConflictError
-                    raise
+                    raise perr
                 if c.description is not None:
                     nselects = nselects + 1
                     if c.description != desc and nselects > 1:
@@ -212,36 +211,15 @@ class DB(TM,dbi_db.DB):
                               'Multiple select schema are not allowed'
                     if max_rows:
                         result = c.fetchmany(max_rows)
+                    else:
+                        result = c.fetchall()
                     desc = c.description
-            failures = 0
-        except psycopg.OperationalError, err:
-            try:
-                del c ; del self.db
-                self.db = self.connect(self.connection)
-            except:
-                raise sys.exc_type, sys.exc_value, sys.exc_traceback
-            failures = 0
-            return self.query(query_string, max_rows)
-        except psycopg.InterfaceError, err:
-            if hasattr(self, 'db'):
-                del c ; del self.db
-                self.db = self.connect(self.connection)
-                return self.query(query_string, max_rows)
-            else:
-                raise sys.exc_type, sys.exc_value, sys.exc_traceback
-        except psycopg.Error, err:
-            failures = failures + 1
-            if (failures > 1000):
-                failures = 0
-                try:
-                    del c ; del self.db
-                    self.db = self.connect(self.connection)
-                except:
-                    raise sys.exc_type, sys.exc_value, sys.exc_traceback
-                return self.query(query_string, max_rows)
-            else:
-                raise sys.exc_type, sys.exc_value, sys.exc_traceback
-
+            self.failures = 0
+            
+        except StandardError, err:
+            self._abort()
+            raise err
+        
         items = []
         for name, typ, width, ds, p, scale, null_ok in desc:
             if typ == NUMBER:
@@ -269,3 +247,6 @@ class DB(TM,dbi_db.DB):
         self.db.close()
         self.db = None
         
+    def sortKey(self):
+        """Zope 2.6 added this one."""
+        return 1
