@@ -21,12 +21,11 @@
 
 /*
  * connection.c -- defines the connection object DBAPI 2.0
- * $Id: connection.c,v 1.51 2003/03/06 10:13:15 fog Exp $
+ * $Id: connection.c,v 1.51.2.5 2004/02/18 20:30:57 fog Exp $
  */
 
-#include <assert.h>
 #include "module.h"
-
+#include <assert.h>
 
 /**** UTILITY FUNCTIONS ****/
 
@@ -51,33 +50,44 @@ curs_closeall(connobject *self)
 }
 
 
-/* curs_commitall() - execute a commit on all the cursors */
-static void
-curs_commitall(connobject *self)
+
+/* _curs_doall() - execute an operation (commit or rollback) on all the cursors 
+ * 
+ * returns NULL on success, a python dictionary if at least one cursor failed.
+ * the dictionary maps key: cursor/cursobject to value:
+ * error/PyString.
+ * Note: in case the dictionary could not be created (out of memory e.g.),
+ *       Py_None is returned instead.
+ */
+static PyObject*
+_curs_doall(connobject *self, int (*operation)(cursobject *) )
 {
-    int len, i;
-    PyObject *cursors = self->cursors;
+    int len, i, has_errors = 0;
     cursobject *cursor;
-    
-    Dprintf("curs_commitall(): acquiring lock\n");
+
+    PyObject* cursors = self->cursors;
+    PyObject* errs = PyDict_New();
+
+    Dprintf("curs_doall: acquiring lock\n");
     pthread_mutex_lock(&(self->lock));
-    Dprintf("curs_commitall(): lock acquired\n");
+    Dprintf("curs_doall: lock acquired\n");
     Py_BEGIN_ALLOW_THREADS;
     
     len = PyList_Size(cursors);
-    Dprintf("curs_commitall(): commit on %d cursors\n", len);
+    Dprintf("curs_doall: %d cursors\n", len);
     
     /* acquire all the required locks */
     for (i = 0; i < len; i++) {
         cursor = (cursobject *)PyList_GetItem(cursors, i);
         assert(cursor);
+        Dprintf("curs_doall: lock/iterating on %p\n", cursor);
         if (cursor->keeper->status == KEEPER_BEGIN
             && cursor->isolation_level > 0){
             pthread_mutex_lock(&(cursor->keeper->lock));
             if (cursor->keeper->status == KEEPER_BEGIN) {
                 cursor->keeper->status = KEEPER_CONN_LOCK;
-                Dprintf("curs_commitall(): acquired lock on keeper %p\n",
-                        cursor->keeper);
+                Dprintf("curs_doall: acquired lock on keeper %p cursor %p\n",
+                        cursor->keeper, cursor);
             }
             else {
                 pthread_mutex_unlock(&(cursor->keeper->lock));
@@ -85,13 +95,24 @@ curs_commitall(connobject *self)
         }
     }
 
-    /* does all the commits */
+    /* does all the operations */
     for (i = 0; i < len; i++) {
+        int status = 0;
         cursor = (cursobject *)PyList_GetItem(cursors, i);
         assert(cursor);
+        Dprintf("curs_doall: iterating on %p\n", cursor);
         if (cursor->keeper->status == KEEPER_CONN_LOCK) {
+            Dprintf("curs_doall: operating on cursor %p\n", cursor);
             cursor->keeper->status = KEEPER_BEGIN;
-            commit_pgconn(cursor);
+            status = (*operation)(cursor);
+            if (status == -1) {
+                has_errors = 1;
+                if (errs) {
+                    PyObject* str = PyString_FromString(cursor->critical);
+                    PyDict_SetItem(errs, (PyObject *)cursor, str);
+                    Py_XDECREF(str);
+                }
+            }
             cursor->keeper->status = KEEPER_CONN_READY;
         }
     }
@@ -103,74 +124,55 @@ curs_commitall(connobject *self)
         if (cursor->keeper->status == KEEPER_CONN_READY) {
             pthread_mutex_unlock(&(cursor->keeper->lock));
             cursor->keeper->status = KEEPER_READY;
-            Dprintf("curs_commitall(): released lock on keeper %p\n",
+            Dprintf("curs_doall: released lock on keeper %p\n",
                     cursor->keeper);
         }
     }
 
     pthread_mutex_unlock(&(self->lock));
-    Dprintf("curs_commitall(): lock released\n");
+    Dprintf("curs_doall: lock released\n");
     Py_END_ALLOW_THREADS;
+
+    if (has_errors) {
+        if (errs)
+            return errs;
+        else {
+            Py_INCREF(Py_None);
+            return Py_None;
+        }
+    }
+    else {
+        Py_DECREF(errs);
+        return NULL;
+    }
 }
 
 
-/* curs_rollbackall() execute a rollback on all the cursors */
-static void
+/* curs_rollbackall() execute a rollback on all the cursors
+ * returns NULL on success, a python dictionary if at least one cursor failed
+ * to rollback; the dictionary maps key: cursor/cursobject to value:
+ * error/PyString.
+ * Note: in case the dictionary could not be created (out of memory e.g.),
+ *       Py_None is returned instead.
+ */
+static PyObject*
 curs_rollbackall(connobject *self)
 {
-    int len, i;
-    PyObject *cursors = self->cursors;
-    cursobject *cursor;
-    
-    Dprintf("curs_rollbackall(): acquiring lock\n");
-    pthread_mutex_lock(&(self->lock));
-    Dprintf("curs_rollbackall(): lock acquired\n");
-    Py_BEGIN_ALLOW_THREADS;
-    
-    len = PyList_Size(cursors);
+    return _curs_doall(self, abort_pgconn);
+}
 
-    /* acquire all the required locks */
-    for (i = 0; i < len; i++) {
-        cursor = (cursobject *)PyList_GetItem(cursors, i);
-        assert(cursor);
-        if (cursor->keeper->status == KEEPER_BEGIN
-            && cursor->isolation_level > 0){
-            pthread_mutex_lock(&(cursor->keeper->lock));
-            if (cursor->keeper->status == KEEPER_BEGIN) {
-                cursor->keeper->status = KEEPER_CONN_LOCK;
-                Dprintf("curs_commitall(): acquired lock on keeper %p\n",
-                        cursor->keeper);
-            }
-            else {
-                pthread_mutex_unlock(&(cursor->keeper->lock));
-            }
-         }
-    }
 
-    /* does all the rollbacks */
-    for (i = 0; i < len; i++) {
-        cursor = (cursobject *)PyList_GetItem(cursors, i);
-        assert(cursor);
-        if (cursor->keeper->status == KEEPER_CONN_LOCK) {
-            cursor->keeper->status = KEEPER_BEGIN;
-            abort_pgconn(cursor);
-            cursor->keeper->status = KEEPER_CONN_BEGIN;
-        }
-    }
-
-    /* unlocks all the connections */
-    for (i = 0; i < len; i++) {
-        cursor = (cursobject *)PyList_GetItem(cursors, i);
-        assert(cursor);
-        if (cursor->keeper->status == KEEPER_CONN_BEGIN) {
-            pthread_mutex_unlock(&(cursor->keeper->lock));
-            cursor->keeper->status = KEEPER_READY;
-        }
-    }
-
-    pthread_mutex_unlock(&(self->lock));
-    Dprintf("curs_rollbackall(): lock released\n");
-    Py_END_ALLOW_THREADS;
+/* curs_commitall() execute a commit on all the cursors
+ * returns NULL on success, a python dictionary if at least one cursor failed
+ * to rollback; the dictionary maps key: cursor/cursobject to value:
+ * error/PyString.
+ * Note: in case the dictionary could not be created (out of memory e.g.),
+ *       Py_None is returned instead.
+ */
+static PyObject*
+curs_commitall(connobject *self)
+{
+    return _curs_doall(self, commit_pgconn);
 }
 
 
@@ -258,36 +260,93 @@ psyco_conn_close(connobject *self, PyObject *args)
 /* psyco_conn_commit() - commit the connection (commit all the cursors) */
 
 static char psyco_conn_commit__doc__[] =
-"Commit any pending transaction to the database.";
+"Commit any pending transaction to the database."
+"Raises DatabaseError in case some cursors fail to commit their changes "
+"--in that case, the value attached to the exception is a dictionary mapping "
+"the cursors that failed with the corresponding error string.";
 
 static PyObject *
 psyco_conn_commit(connobject *self,PyObject *args)
 {
+    PyObject* errs;
+   
     EXC_IFCLOSED(self);
     PARSEARGS(args);
-    curs_commitall(self);
-    Dprintf("psyco_conn_commit(): commit executed\n");
-    Py_INCREF(Py_None);
-    return Py_None;
+
+    errs = curs_commitall(self);
+
+    if (errs) {
+        cursobject* cursor;
+        PyObject *key, *value;
+        int pos = 0;
+        Dprintf("psyco_conn_commit: curs_commitall() failed\n");
+        PyErr_SetObject(DatabaseError, errs);
+        if (errs != Py_None) {
+            /* Now that the errors have been taken into account, make sure
+               that the cursors that were marked as critical can be used again
+               in the future. There is no need to set the keeper's status,
+               this has been done in curs_commitall() */
+            while (PyDict_Next(errs, &pos, &key, &value)) {
+            	cursor = (cursobject*)key;
+            	if (cursor->critical) free(cursor->critical);
+            	cursor->critical = NULL;
+            }
+        }
+        Py_DECREF(errs);
+        return NULL;
+    }
+    else {
+        Dprintf("psyco_conn_commit: commit executed\n");
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
 }
 
 
 /* psyco_conn_rollback() - rollback the connection (rollback on all cursors) */
 
 static char psyco_conn_rollback__doc__[] = 
-"Causes the database to roll back to the start of any pending transaction.";
+"Causes the database to roll back to the start of any pending transaction."
+"Raises DatabaseError in case some cursors fail to rollback their changes "
+"--in that case, the value attached to the exception is a dictionary mapping "
+"the cursors that failed with the corresponding error string.";
 
 static PyObject *
 psyco_conn_rollback(connobject *self,	PyObject *args)
 {
+    PyObject* errs;
+   
     EXC_IFCLOSED(self);
     PARSEARGS(args);
-    curs_rollbackall(self);
-    Dprintf("psyco_conn_rollback(): rollback executed\n");
-    Py_INCREF(Py_None);
-    return Py_None;
-}
 
+    errs = curs_rollbackall(self);
+    
+    if (errs) {
+        cursobject* cursor;
+        PyObject *key, *value;
+        int pos = 0;
+        Dprintf("psyco_conn_rollback: curs_rollbackall() failed\n");
+        PyErr_SetObject(DatabaseError, errs);
+        if (errs!=Py_None) {
+            /* Now that the errors have been taken into account, make sure
+               that the cursors that were marked as critical can be used again
+               in the future. There is no need to set the keeper's status,
+               this has been done in curs_rollbackall() */
+            while (PyDict_Next(errs, &pos, &key, &value)) {
+            	cursor = (cursobject*)key;
+            	if (cursor->critical) free(cursor->critical);
+            	cursor->critical = NULL;
+            }
+        }
+        Py_DECREF(errs);
+        return NULL;
+    }
+    else {
+        Dprintf("psyco_conn_rollback(): rollback executed\n");
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+}
 
 /* psyco_conn_cursor() - create a new cursor */
 
@@ -341,16 +400,25 @@ _psyco_conn_set_isolation_level(connobject *self, int level)
     PyObject *tmpobj;
     int i, len;
     
-   /* save the value */
-    self->isolation_level = (int)level;
+    /* save the value */
+    if (level < 0) level = 0;
+    if (level > 3) level = 3;
+    self->isolation_level = level;
+
+    /* make sure the list of cursors will not change by other threads while
+       iterating (curs_switch_isolation_level() allows other threads to run) */
+    pthread_mutex_lock(&(self->lock));
 
     /* call set_isolation_level on all the cursors to change their status */
     len = PyList_Size(self->cursors);
     for (i = 0; i < len; i++) {
         tmpobj = PyList_GetItem(self->cursors, i);
         assert(tmpobj);
+        Py_INCREF(tmpobj);
         curs_switch_isolation_level((cursobject *)tmpobj, level);
+        Py_DECREF(tmpobj);
     }
+    pthread_mutex_unlock(&(self->lock));
 }
 
 static PyObject *
@@ -361,7 +429,7 @@ psyco_conn_set_isolation_level(connobject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "l", &level)) {
         return NULL;
     }
-
+    
     _psyco_conn_set_isolation_level(self, level);
     
     Py_INCREF(Py_None);
