@@ -20,18 +20,12 @@
 
 /*
  * typemod.c -- types available to the psycopg module
- * $Id: typemod.c,v 1.13 2003/05/07 13:27:17 fog Exp $
+ * $Id: typemod.c,v 1.13.2.2 2003/11/20 23:53:46 fog Exp $
  */
 
-#ifndef _WIN32
-#include <unistd.h>
-#endif
-
-#include <assert.h>
-
 #include "module.h"
 #include "typemod.h"
-
+#include <assert.h>
 
 /**** DateTimeObject OBJECT DEFINITIONS ****/
 
@@ -443,6 +437,10 @@ new_psyco_quotedstringobject(PyObject *buffer)
             quoted[j++] = '\\';
             break;
 
+        case '\0':
+            /* do nothing, embedded \0 are discarded */
+            break;
+            
         default:
             quoted[j++] = original[i];
         }
