@@ -20,12 +20,12 @@
 
 /*
  * typeobj.c -- define the type objects
- * $Id: typeobj.c,v 1.24 2003/03/06 10:13:15 fog Exp $
+ * $Id: typeobj.c,v 1.24.2.3 2004/05/16 08:09:49 fog Exp $
  */
 
-#include "assert.h"
 #include "module.h"
 #include "typeobj.h"
+#include <assert.h>
 
 static PyObject *psyco_DBAPITypeObject_new(PyObject *, PyObject *, PyObject *);
 extern PyTypeObject psyco_DBAPITypeObject_Type;
@@ -122,7 +122,7 @@ psyco_TIME_cast(PyObject *s)
         return NULL;
     }
 
-    return mxDateTimeP->DateTime_FromDateAndTime(1970, 1, 1, hh, mm ,ss);
+    return mxDateTimeP->DateTimeDelta_FromTime(hh, mm ,ss);
 }
 
 static char *
@@ -239,7 +239,7 @@ psyco_INTERVAL_cast(PyObject *s)
     }
     
     /* calculates seconds */
-    if (hours < 0.0) {
+    if (hours < 0.0 || hours == -0.0) {
         seconds = - (hundredths + seconds + minutes*60) + hours*3600;
     }
     else {
